import Foundation

/**
 States the FirmwareUpdatePresenter uses to handle the firmware update process.
 */
enum FirmwareUpdatePresenterState : CustomDebugStringConvertible, Equatable {
    /// Initial state, waiting for a firmware update to become available.
    case start
    /// Firmware update available, waiting for firmware update to be triggered.
    case updateAvailable
    /// Firmware update process started, connecting to backend.
    case connectingToBackend
    /// Connection established, downloading firmware update.
    case downloading(_ percent:Int)
    /// Uploading firmware update to device.
    case uploading(_ percent:Int)
    /// Waiting for device to install firmware and reboot.
    case rebooting
    /// Final state, firmware update finished successfully if `error` was `nil, otherwise the firmware update failed.
    case finished(error:FirmwareUpdateError?)

    var debugDescription: String {
        switch self {
        case .start: return "initialised"
        case .updateAvailable: return "updateAvailable"
        case .connectingToBackend: return "connectingToBackend"
        case .downloading(let percent): return "downloading \(percent)%"
        case .uploading(let percent): return "uploading \(percent)%"
        case .rebooting: return "rebooting"
        case .finished(let error): return "finished with error " + (error == nil ? "nil" : "\(String(describing: error))")
        }
    }

    private var rawValue : Int {
        get {
            switch self {
            case .start: return 1
            case .updateAvailable: return 2
            case .connectingToBackend: return 3
            case .downloading: return 4
            case .uploading: return 5
            case .rebooting: return 6
            case .finished(_): return 7
            }
        }
    }

    static func ==(lhs: FirmwareUpdatePresenterState, rhs: FirmwareUpdatePresenterState) -> Bool {
        // ignore associated values
        return lhs.rawValue == rhs.rawValue
    }
}
