import UIKit
import os

/**
 ViewController handling the main sceen, showing information and handling input for:
 
 - device and backend connection status, see DeviceStatusPresenter
 - synchronizing device data with the backend, see SynchronizationPresenter
 - real time data transmission from the device, see RealTimeDataPresenter
 - firmware update of the device, see FirmwareUpdatePresenter
 
 */
class MainViewController: UIViewController, DeviceStatusPresenterDelegate, SynchronizationPresenterDelegate, RealTimeDataPresenterDelegate, FirmwareUpdatePresenterDelegate {
    
    private let userDao = ServiceFactory.userService
    private let deviceStatusPresenter = DeviceStatusPresenter(
        connectionService: ServiceFactory.connectionService, userService: ServiceFactory.userService)
    private let synchronizationPresenter = SynchronizationPresenter(
        serviceClient: ServiceFactory.serviceClient, userService: ServiceFactory.userService, connectionService: ServiceFactory.connectionService)
    private let realTimeDataPresenter = RealTimeDataPresenter(
        serviceClient: ServiceFactory.serviceClient, userService: ServiceFactory.userService)
    private let firmwareUpdatePresenter = FirmwareUpdatePresenter(
        serviceClient: ServiceFactory.serviceClient, userService: ServiceFactory.userService, connectionService: ServiceFactory.connectionService)
   
    @IBOutlet weak var deviceStatusView: UIView!
    @IBOutlet weak var serverStatusView: UIView!
    @IBOutlet weak var serverStatusLabel: UILabel!
    @IBOutlet weak var deviceStatusLabel: UILabel!

    @IBOutlet weak var syncProgressView: UIProgressView!
    @IBOutlet weak var syncStatusLabel: UILabel!
    @IBOutlet weak var syncButton: UIButton!
    
    @IBOutlet weak var realTimeDataStartButton: UIButton!
    @IBOutlet weak var realTimeDataStopButton: UIButton!
    @IBOutlet weak var odometerValueLabel: UILabel!
    @IBOutlet weak var rpmValueLabel: UILabel!
    
    @IBOutlet weak var firmwareUpdateStatusLabel: UILabel!
    @IBOutlet weak var firmwareUpdateProgressView: UIProgressView!
    @IBOutlet weak var firmwareUpdateButton: UIButton!
    
    //
    // MARK: - Controller Life Cycle
    //
    
    override func viewDidLoad() {
        super.viewDidLoad()
        deviceStatusPresenter.delegate = self
        synchronizationPresenter.delegate = self
        realTimeDataPresenter.delegate = self
        firmwareUpdatePresenter.delegate = self
        updateRealTimeDataButtons()
        updateRealTimeValues()
        updateDeviceStatus()
        updateFirmwareStatus()
    }
    
    //
    // MARK: - Logout
    //
    
    @IBAction func onLogout(_ sender: Any) {
        userDao.deleteCredentials()
        performSegue(withIdentifier: "showLoginScreen", sender: self)
    }
    
    //
    // MARK: - Device Status
    //
    
    func onConnectionStateChanged() {
        updateDeviceStatus()
    }
    
    func updateDeviceStatus()
    {
        deviceStatusLabel.text = deviceStatusPresenter.deviceConnectionStatus
        serverStatusLabel.text = deviceStatusPresenter.serverConnectionStatus
        deviceStatusView.backgroundColor = deviceStatusPresenter.deviceConnected ? UIColor.green : .lightGray
        serverStatusView.backgroundColor = deviceStatusPresenter.serverConnected ? UIColor.green : .lightGray
    }
    
    //
    // MARK: - Synchronization
    //
    
    @IBAction func startSync(_ sender: Any) {
        synchronizationPresenter.startSync()
    }
    
    /// Sync in progress. create an overall progress by assigning each state a progress value.
    /// `downloading` provides it's own progress which scalled to progress between `0.2` to `0.9`.
    func onSyncStateChange(state: SynchronizationPresenterState) {
        os_log("onSyncStateChange %s", log: createTraceLog(), String(reflecting: state))
        syncProgressView.alpha = 1
        var buttonEnabled = false
        switch state {
        case .connectingToBackend:
            syncProgressView.progress = 0.1
        case .clockSynchronization:
            syncProgressView.progress = 0.15
        case .downloading(let progress):
            syncProgressView.progress = 0.2 + Float(progress) / 100 * 0.7
        case .uploading:
            syncProgressView.progress = 0.9
        case .finished:
            syncProgressView.progress = 1
            UIView.animate(withDuration: 0.5) {
                self.syncProgressView.alpha = 0
            }
            buttonEnabled = true
        case .start:
            buttonEnabled = true
        }
        syncButton.isEnabled = buttonEnabled
        syncButton.backgroundColor = buttonEnabled ? Colors.defaultColor : Colors.disabled
        syncStatusLabel.text = state.toLocalizedString(progress: syncProgressView.progress)
    }
    
    /// The device was unplugged since the last sync the user needs to provide a reason why the device was unplugged.
    func onClockSyncReasonRequired() {
        let alertController = UIAlertController(
            title: localize(key: "MainViewController.disconnectAlertTitle"),
            message: localize(key: "MainViewController.disconnectAlertMessage"),
            preferredStyle: .actionSheet
        )
        ClockSyncReason.allCases.forEach { (reason) in
            alertController.addAction(UIAlertAction(title: reason.toLocalizedString(), style: .default, handler: { (action) in
                self.synchronizationPresenter.sendClockSyncReason(reason)
            }))
        }
        self.present(alertController, animated: true)
    }
    
    //
    // MARK: - Realtime Data
    //
    
    @IBAction func startRealTimeData(_ sender: Any) {
        realTimeDataPresenter.startRealTimeData()
        updateRealTimeDataButtons()
    }
    
    @IBAction func stopRealtimeData(_ sender: Any) {
        realTimeDataPresenter.stopRealTimeData()
        updateRealTimeDataButtons()
    }
    
    func onDatagramChanged() {
        updateRealTimeValues()
    }
    
    private func updateRealTimeDataButtons(){
        realTimeDataStartButton.isEnabled = !realTimeDataPresenter.enabled
        realTimeDataStartButton.backgroundColor = realTimeDataPresenter.enabled ? Colors.disabled : Colors.defaultColor
        realTimeDataStopButton.isEnabled = realTimeDataPresenter.enabled
        realTimeDataStopButton.backgroundColor = realTimeDataPresenter.enabled ? Colors.defaultColor : Colors.disabled
    }
    
    private func updateRealTimeValues() {
        odometerValueLabel.text = realTimeDataPresenter.odometer
        rpmValueLabel.text = realTimeDataPresenter.rpm
    }
    
    //
    // MARK: - Firmware Update
    //
    
    /// get firmware update status from presenter and update view.
    private func updateFirmwareStatus() {
        let state = firmwareUpdatePresenter.state
        var progress = 0
        var enabled = false
        switch firmwareUpdatePresenter.state {
        case .updateAvailable:
            enabled = true
        case .downloading(let percent), .uploading(let percent):
            progress = percent
        default:
            break
        }
        updateFirmwareStatusViews(state.toLocalizedString(), progress: progress, enabled: enabled)
    }
    
    private func updateFirmwareStatusViews(_ statusText: String, progress: Int, enabled: Bool) {
        firmwareUpdateButton.isEnabled = enabled
        firmwareUpdateButton.backgroundColor = enabled ? Colors.defaultColor : Colors.disabled
        firmwareUpdateStatusLabel.text = statusText
        if progress > 0 {
            firmwareUpdateProgressView.alpha = 1.0
            firmwareUpdateProgressView.progress = Float(progress) / Float(100.0)
        } else {
            firmwareUpdateProgressView.alpha = 0.0
            firmwareUpdateProgressView.progress = 0
        }
    }
    
    @IBAction func onStartFirmwareUpdate(_ sender: Any) {
        firmwareUpdatePresenter.startFirmwareUpdate()
    }
    
    func onFirmwareUpdateStateChange() {
        updateFirmwareStatus()
    }
    
}
