//
// Copyright (c) 2013 - 2015 TomTom Telematics. All rights reserved.
//


#import <Foundation/Foundation.h>

#define TTL_SERVICE_ERROR_DOMAIN_DEVICE @"TTLServiceErrorDomainDevice"
#define TTL_SERVICE_ERROR_DOMAIN_SDK @"TTLServiceErrorDomainSDK"
#define TTL_SERVICE_ERROR_DOMAIN_SERVER @"TTLServiceErrorDomainServer"

#define TTL_SERVICE_ERROR_CODE_ACTIVATION_FAILED 1
#define TTL_SERVICE_ERROR_CODE_SYNC_FAILED_CONNECTION_NEEDED 2
#define TTL_SERVICE_ERROR_CODE_SYNC_FAILED_RESPONSE_DELAYED 3
#define TTL_SERVICE_ERROR_CODE_SYNC_FAILED_REASON_NEEDED 4

/** The connection failed because of a network problem. */
#define TTL_SERVICE_ERROR_CODE_SERVER_CONNECTION_FAILED 5
/** The wrog username or password. */
#define TTL_SERVICE_ERROR_CODE_SERVER_CONNECTION_WRONG_CREDENTIALS 6
/** The device identifier is not associated with the credentials. */
#define TTL_SERVICE_ERROR_CODE_SERVER_CONNECTION_WRONG_DEVICE_ID 7

typedef NS_ENUM(NSInteger, TTLServiceClockSyncAction)
{
    TTLServiceClockSyncForDownload = 1,
    TTLServiceClockSyncForDeviceSetup = 2
};

/** Download progress states.
*
* Used with TTLServiceDownloadStatusResponse of [TTLServiceEventDelegate onDownloadStatusUpdate:response:].
*
* @see TTLServiceDownloadStatusResponse
* @see [TTLServiceEventDelegate onDownloadStatusUpdate:response:]
*/
typedef NS_ENUM(NSInteger, TTLServiceDownloadProgressStatus)
{
    TTLServiceDownloadStarted = 1,
    TTLServiceDownloadProgress = 2,
    TTLServiceDownloadSuccessful = 3,
    TTLServiceDownloadFailed = 4, // Returned when there is a problem with syncing in general (so download from dongle AND upload to the backend)
    TTLServiceUploadSuccessful = 5
};

typedef NS_ENUM(NSInteger, TTLServiceFirmwareUpdateStatus)
{
    // States for downloading the firmware from backend.
    TTLServiceFirmwareDownloadStarted = 1,
    TTLServiceFirmwareDownloadProgress = 1 << 1,
    TTLServiceFirmwareDownloadFailed = 1 << 2,
    TTLServiceFirmwareDownloadFinished = 1 << 3,

    // States for uploading the firmware to the LINK100 device.
    TTLServiceFirmwareUploadStarted = 1 << 4,
    TTLServiceFirmwareUploadProgress = 1 << 5,
    TTLServiceFirmwareUploadFailed = 1 << 6,
    TTLServiceFirmwareUploadFinished = 1 << 7,

    TTLServiceFirmwareInstallRequested = 1 << 8
};

typedef NS_ENUM(NSInteger, TTLServiceOdometerChangeStatus)
{
    TTLServiceOdometerChangeSuccessful = 1,
    TTLServiceOdometerChangeDeniedResponseDelayed = 2,
    TTLServiceOdometerChangeFailed = 3
};

typedef NS_ENUM(NSInteger, TTLServiceStateDeviceActivation)
{
    TTLServiceStateDeviceActivated = 1,
    TTLServiceStateDeviceNotActivated = 2,
    TTLServiceStateDeviceActivationUnknown = 3
};

typedef NS_ENUM(NSInteger, TTLServiceStateServerConnection)
{
    TTLServiceStateServerConnected = 1,
    TTLServiceStateServerNotConnected = 2,
    TTLServiceStateServerConnectionUnknown = 3
};

typedef NS_ENUM(NSInteger, TTLServiceStateDeviceConnection)
{
    TTLServiceStateDeviceConnected = 1,
    TTLServiceStateDeviceNotConnected = 2,
    TTLServiceStateDeviceConnectionUnknown = 3
};


typedef NS_ENUM(NSInteger, TTLServiceTransmissionType)
{
    TTLServiceTransmissionToDevice = 1,
    TTLServiceTransmissionFromDevice = 2,
    TTLServiceTransmissionToServer = 3,
    TTLServiceTransmissionFromServer = 4
};


#pragma mark - TTLServiceEventDelegate

/** Reason why activation process failed.
 *
 * The reason is delivered as a NSError which contains a localized string in the userInfo dictionary
 * ('NSLocalizedDescriptionKey' and 'NSLocalizedFailureReasonErrorKey' are supported).
 *
 */
@interface TTLServiceActivationFailedReason : NSObject
/** A NSError which contains the reason for the failure.
 *
 * The reason can be ether in the device domain ('TTL_SERVICE_ERROR_DOMAIN_DEVICE') or in the SDK
 * ('TTL_SERVICE_ERROR_DOMAIN_SDK'). For example the LINK device or the backend services are not
 * connected. Descriptions are provided in the userInfo dictionary ('NSLocalizedDescriptionKey' and
 * 'NSLocalizedFailureReasonErrorKey' are supported).
 *
 * @see [TTLServiceEventDelegate onActivationFailed:reason:]
 */
@property(nonatomic) NSError * _Nonnull error;

- (NSString * _Nonnull)description;
@end


/** Reason why clock sync process failed.
 *
 * The reason is delivered as a NSError which contains a localized string in the userInfo dictionary
 * ('NSLocalizedDescriptionKey' and 'NSLocalizedFailureReasonErrorKey' are supported).
 *
 */
@interface TTLServiceClockSyncFailedReason : NSObject

/** A NSError which contains the reason for the failure.
 *
 * The reason can be either in the device domain ('TTL_SERVICE_ERROR_DOMAIN_DEVICE') or in the SDK
 * ('TTL_SERVICE_ERROR_DOMAIN_SDK'). For example if the LINK device or the backend services are not
 * connected. Descriptions are provided in the userInfo dictionary ('NSLocalizedDescriptionKey' and
 * 'NSLocalizedFailureReasonErrorKey' are supported).
 *
 * @see [TTLServiceEventDelegate onActivationFailed:reason:]
 */
@property(nonatomic) NSError * _Nonnull error;

- (NSString * _Nonnull)description;
@end

/** Encapsulates the cause of the successful synchronisation of the clock.
 *
 * The clock synchronisation is either initiated automatically by the LINK device on certain
 * actions like downloading driving data (see [TTLServiceEventDelegate onClockSyncReasonWhileDownloadRequired:])
 * or initiated by the custom app when calling [TTLServiceClient sendClockSyncReasonForDeviceSetup:deviceIdentifier:].
 *
 * @see [TTLServiceEventDelegate onClockSyncSuccessful:reason:].
 */
@interface TTLServiceClockSyncForActionResponse : NSObject

/** The action in which the sync was initiated.
 *
 */
@property(nonatomic) TTLServiceClockSyncAction action;

- (NSString * _Nonnull)description;
@end

/** Contains crash detection data.
 * 
 * Used with TTLServiceCrashDatagramResponse.
 *
 * @see TTLServiceCrashDatagramResponse
 */
@interface TTLServiceCrashDatagram : NSObject
/**
 * The crash event time. UTC, date.
 */
@property(nonatomic) NSDate * _Nullable eventTime;

/**
 * The acceleration when the crash was detected, in mg, between 0 and 65535, Integer.
 */
@property(nonatomic) NSNumber * _Nullable acceleration;

/**
 * The direction of the acceleration when the crash was detected, in degree, between 0 and 359, Integer.
 */
@property(nonatomic) NSNumber * _Nullable accelerationDirection;

/**
 * Virtual odometer in m. Long.
 */
@property(nonatomic) NSNumber * _Nullable odometer;

/**
 *   Vehicle speed. Integer. In km/h, between 0 km/h and 1000 km/h.
 */
@property(nonatomic) NSNumber * _Nullable vehicleSpeed;

/**
 * Engine RPM. Integer.
 */
@property(nonatomic) NSNumber * _Nullable engineRPM;

/**
 *Throttle position. Integer, in percent, between 0% and 100%.
 */
@property(nonatomic) NSNumber * _Nullable throttlePosition;

/**
 * Calculated engine load. Integer, in percent, between 0% and 100%.
 */
@property(nonatomic) NSNumber * _Nullable engineLoad;

- (NSString * _Nonnull)description;
@end

/** Contains a datagram containing crash related data received via [TTLServiceEventDelegate onCrashDetected:response:].
 *
 */
@interface TTLServiceCrashDatagramResponse : NSObject
@property(nonatomic) TTLServiceCrashDatagram * _Nonnull crashDatagram;

- (NSString * _Nonnull)description;
@end

/** Encapsulates a arbitrary reason for why a clock sync is required.
 *
 * Used with [TTLServiceClient sendClockSyncReasonForDeviceSetup:deviceIdentifier:] and [TTLServiceClient sendClockSyncReasonForDownload:deviceIdentifier:].
 *
 * @see [TTLServiceClient sendClockSyncReasonForDeviceSetup:deviceIdentifier:]
 * @see [TTLServiceClient sendClockSyncReasonForDownload:deviceIdentifier:]
 */
@interface TTLServiceDeviceUnplugReason : NSObject
@property(nonatomic) int reason;

- (instancetype _Nonnull)initWithReason:(int)reason;

- (NSString * _Nonnull)description;
@end

/** Collects information about the LINK device.
 *
 */
@interface TTLServiceDeviceVersionInfo : NSObject
/**
 * The LINK device's hardware version.
 */
@property(nonatomic) NSNumber * _Nullable hardwareVersion;

/**
 * The major version of the loader on the LINK device.
 */
@property(nonatomic) NSNumber * _Nullable loaderMajorVersion;

/**
 * The minor version of the loader on the LINK device.
 */
@property(nonatomic) NSNumber * _Nullable loaderMinorVersion;

/**
 * The build number of the loader on the LINK device.
 */
@property(nonatomic) NSNumber * _Nullable loaderBuild;

/**
 * The major version of the UBI app on the LINK device.
 */
@property(nonatomic) NSNumber * _Nullable applicationMajorVersion;

/**
 * The minor version of the UBI app on the LINK device.
 */
@property(nonatomic) NSNumber * _Nullable applicationMinorVersion;

/**
 * The build number of the UBI app on the LINK device.
 */
@property(nonatomic) NSNumber * _Nullable applicationBuild;

/**
 * Returns a readable representation of the versions of the UBI app on the LINK device.
 *
 * Do not use this version text for internal processing but only for the purpose of displaying.
 */
- (NSString * _Nonnull)readableApplicationVersion;

/**
 * Returns a readable representation of the versions of the UBI app on the LINK device.
 *
 * Do not use this version text for internal processing but only for the purpose of displaying.
 */
- (NSString * _Nonnull)readableLoaderVersion;

- (NSString * _Nonnull)description;
@end

/** Encapsulates the LINK device state.
*
* This is delivered when the LINK device is asked for it's state. State, in this regards, means
* basically the odometer reading of the LINK device and the total number of times the LINK device was disconnected
* (or lost power).
*
*  See [TTLServiceEventDelegate onDeviceState:response:]
*
*/
@interface TTLServiceDeviceStateResponse : NSObject

/// The odometer reading of the LINK device.
@property(nonatomic) NSNumber * _Nullable odometer;

/// Total number of times the LINK device was disconnected (or lost power).
@property(nonatomic) NSNumber * _Nullable disconnectCount;
@end


/** Encapsulates the current progress after calling [TTLServiceClient startSyncForDevice:].
 *
 *  See [TTLServiceEventDelegate onDownloadStatusUpdate:response:] and TTLServiceDownloadProgressStatus
 */
@interface TTLServiceDownloadStatusResponse : NSObject

/** The state of the download which will be one the options of `TTLServiceDownloadProgressStatus`
 *
 * Note, the states `TTLServiceDownloadSuccessful` or `TTLServiceDownloadFailed`
 * do NOT indicate that the upload to the server has finished or failed. The process of uploading
 * the driving data to the server is handled transparently by the SDK and the progress cannot be monitored explicitly.
 *
 * However, when all upload data arrived at the backend successfully [TTLServiceEventDelegate onDownloadStatusUpdate:response:]
 * will be called with the progressState `TTLServiceUploadSuccessful`
 *
 * @see [TTLServiceEventDelegate onDownloadStatusUpdate:response:]
 */
@property(nonatomic) TTLServiceDownloadProgressStatus progressState;

/** Indicates the amount of progress depending on the value `progressState`
 *
 * - In case `progressState` is `TTLServiceDownloadStarted`, `progress` contains the maximum number of bytes that are going to be sent
 * from the LINK device to the smart phone. This information can be used to initialize a progress bar.
 *
 * - In case `progressState` is `TTLServiceDownloadProgress`, `progress` contains the current value of transferred bytes with
 * the last transmission. This value is NOT the cumulated progress - this has to be handled by the custom app itself.
 *
 * - In case `progressState` is `TTLServiceDownloadSuccessful`, `TTLServiceDownloadFailed` or `TTLServiceUploadSuccessful` the progress
 * is <code>nil</code>.
 *
 */
@property(nonatomic) NSNumber * _Nullable progress;

- (NSString * _Nonnull)description;
@end

/**
 * Contains a list of DTCs (OBD Diagnostic Trouble Codes)  received via [TTLServiceEventDelegate onDTCList:response:] which
 * is called after [TTLServiceClient requestDTCList:] was called.
 *
 * @see [TTLServiceClient requestDTCList:]
 */
@interface TTLServiceDTCListResponse : NSObject
/** A array of strings representing DTCs */
@property(nonatomic) NSArray * _Nonnull dtcList;

- (NSString * _Nonnull)description;
@end

/**
 * Contains information about the engine status. Currently it only answers the question if the engine is on or off.
 */
@interface TTLServiceEngineStatusResponse : NSObject

/**
 * True if the engine is on, false otherwise.
 */
@property(nonatomic) BOOL engineOn;
@end

/**
 * Encapsulates the information that a firmware update is available. When appropriate the service asks the server if a
 * new firmware is available. If so, the server sends a message which can be received via [TTLServiceEventDelegate onFirmwareUpdateAvailable:response:].
 * Thereafter an app can call [TTLServiceClient updateFirmware:deviceIdentifier:] to start the actual firmware update.
 *
 * @see [TTLServiceEventDelegate onFirmwareUpdateAvailable:response:]
 */
@interface TTLServiceFirmwareUpdateAvailableResponse : NSObject
@property(nonatomic) long long firmwareFileId;
@property(nonatomic) long firmwareSizeInBytes;
@property(nonatomic) NSString * _Nonnull updateFirmwareVersion;
@property(nonatomic) NSString * _Nonnull currentFirmwareVersion;
@end

/**
 * Encapsulates the current progress after a firmware update has been initiated. A firmware update is announced via
 * [TTLServiceEventDelegate onFirmwareUpdateAvailable:response:].
 *
 * @see [TTLServiceEventDelegate onFirmwareUpdateStatusUpdate:response:].
 */

@interface TTLServiceFirmwareUpdateStatusResponse : NSObject
@property(nonatomic) TTLServiceFirmwareUpdateStatus status;
/**
 * Download/Upload progress from 0 to 100 in percent.
 */
@property(nonatomic) long progress;
@property(nonatomic) long long firmwareUpdateFileId;
@end

/** Encapsulates the result of a call to [TTLServiceClient changeOdometer:deviceIdentifier:]
 *
 * See also [TTLServiceEventDelegate onOdometerChange:response:].
 *
 * @see [TTLServiceEventDelegate onOdometerChange:response:]
 */
@interface TTLServiceOdometerChangeResponse : NSObject
@property(nonatomic) TTLServiceOdometerChangeStatus odometerChangeStatus;

- (NSString * _Nonnull)description;
@end

/** Reason why the connection to the server (backend services) failed.
 *
 * The reason is delivered as a NSError which contains a localized string in the userInfo dictionary
 * ('NSLocalizedDescriptionKey' and 'NSLocalizedFailureReasonErrorKey' are supported).
 *
 * @see [TTLServiceEventDelegate onServerConnectionFailed:reason:]
 */
@interface TTLServiceServerConnectionFailedReason : NSObject

/** A NSError which contains the reason for the failure.
 *
 * Possible error codes:
 * - TTL_SERVICE_ERROR_CODE_SERVER_CONNECTION_WRONG_DEVICE_ID: The device can not be used with that account
 * (maybe the account is already bound to another device or the device itself is bound to another account)
 * - TTL_SERVICE_ERROR_CODE_SERVER_CONNECTION_WRONG_CREDENTIALS: A login with the credentials is not possible (probably
 * a typo)
 * - TTL_SERVICE_ERROR_CODE_SERVER_CONNECTION_FAILED: The connection failed because of network problems.
 */
@property(nonatomic) NSError * _Nonnull error;

- (NSString * _Nonnull)description;
@end

/** Represents the state of the TomTom UBI service for a specific device.
 *
 * This will be provided via [TTLServiceEventDelegate onServiceState:] when [TTLServiceClient requestServiceState] was
 * called or when a state changes occurs.
 */
@interface TTLServiceStateResponse : NSObject
@property(nonatomic) TTLServiceStateDeviceActivation deviceActivation;
@property(nonatomic) TTLServiceStateDeviceConnection deviceConnection;
@property(nonatomic) TTLServiceStateServerConnection serverConnection;
@property(nonatomic) TTLServiceDeviceVersionInfo * _Nullable deviceVersionInfo;
/** Vehicle Identification Number */
@property(nonatomic) NSString * _Nullable vin;
@property(nonatomic) NSString * _Nonnull deviceIdentifier;
@end

/** Indicates what kind of transition was ongoing when [TTLServiceEventDelegate onTransmission:response:] was called.
 *
 * @see [TTLServiceEventDelegate onTransmission:response:]
 */
@interface TTLServiceTransmissionResponse : NSObject
@property(nonatomic) TTLServiceTransmissionType transmissionType;

- (NSString * _Nonnull)description;
@end

/** Callback Delegate Protocol for communication from tomtom ubi service to the custom application.
 *
 * This delegate protocol provides callbacks for any communication from the tomtom UBI service to the custom app.
 * The tomtom UBI service is the piece of software that handles all communication between LINK device and a UBI
 * backend server. It keeps an internal state, takes care of the initial activation of the LINK device and serves as a
 * transport proxy for all data that need to be transmitted between LINK device and backend and vice versa.
 *
 * All callbacks are scheduled on the main queue.
 *
 * @see TTLServiceClient
 * @see TTLServiceOBDRealTimeDelegate
 */
@protocol TTLServiceEventDelegate <NSObject>

//  TODO Docu. Is not required anymore but if not implemented by at least on delegate, the sdk creates its own reason.

@optional
/** Clock sync needed while download driving data was requested.
 *
 * Called when the Service was initiated to download the driving the data from the LINK device but the device requested
 * the synchronisation of the internal clock. The user has to enter a reason for the synchronisation of the clock.
 * This is to track possible disconnects of the LINK device from the vehicle and why they happened.
 *
 * The client has to call [TTLServiceClient sendClockSyncReasonForDownload:deviceIdentifier:] to continue the
 * synchronisation process.
 *
 * @see [TTLServiceClient sendClockSyncReasonForDownload:deviceIdentifier:]
 * @see [TTLServiceClient startSyncForDevice:]
 */
- (void)onClockSyncReasonWhileDownloadRequired:(NSString * _Nonnull)deviceIdentifier;

/** Device activation was started.
 *
 * Called at every start of an activation process. For example this happens if the LINK device was not yet activated
 * or after a firmware update has successfully been performed.
 *
 * This can be used in the client to initialize the UI accordingly.
 */
- (void)onActivationStart:(NSString * _Nonnull)deviceIdentifier;

/** Device activation was finished and was successful.
 *
 * The resulting state of the LINK device activation is also prompted via [TTLServiceEventDelegate onServiceState:]
 *
 * This can be used in the client to initialize the UI accordingly.
 */
- (void)onActivationSuccessful:(NSString * _Nonnull)deviceIdentifier;

/** Device activation was finished but was not was successful.
 *
 * The resulting state of the LINK device activation is also prompted via [TTLServiceEventDelegate onServiceState:]
 *
 * This can be used in the client to initialize the UI accordingly.
 *
 * @see TTLServiceActivationFailedReason
 */
- (void)onActivationFailed:(NSString * _Nonnull)deviceIdentifier reason:(TTLServiceActivationFailedReason * _Nonnull)reason;


/** Event method called when clock Sync was successful.
 *
 * @see TTLServiceClockSyncForActionResponse
 */
- (void)onClockSyncSuccessful:(NSString * _Nonnull)deviceIdentifier reason:(TTLServiceClockSyncForActionResponse * _Nonnull)response;

/** Event method called when clock Sync was unsuccessful.
 *
 */
- (void)onClockSyncFailed:(NSString * _Nonnull)deviceIdentifier reason:(TTLServiceClockSyncFailedReason * _Nonnull)reason;

- (void)onCrashDetected:(NSString * _Nonnull)deviceIdentifier response:(TTLServiceCrashDatagramResponse * _Nonnull)response;

/** Event method called when a LINK device becomes available  (in terms of bluetooth accessories) to service.
 *
 * The contextual information is also provided with [onServiceState:].
 */
- (void)onDeviceDidAppear:(NSString * _Nonnull)deviceIdentifier;

/** Event method called when a LINK device becomes unavailable  (in terms of bluetooth accessories) to service.
 *
 * The contextual information is also provided with [onServiceState:].
 */
- (void)onDeviceDidDisappear:(NSString * _Nonnull)deviceIdentifier;

/** Event method called when a LINK device was requested to deliver it's device state.
*
* @see TTLServiceDeviceStateResponse
* @see [TTLServiceClient requestDeviceState]
*/
- (void)onDeviceState:(NSString * _Nonnull)deviceIdentifier response:(TTLServiceDeviceStateResponse * _Nonnull) response;

/** Event method called when a download (LINK => smart phone => UBI Server) is in progress.
 *
 * The the progress of the transfer between device and smart phone can be monitored with this event.
 * The upload of the data will happen automatically when a connection to the backend service was established.
 * This upload progress cannot be monitored but the end of it will be signaled.
 *
 * See TTLServiceDownloadStatusResponse and its properties on how the different states of the download process are handled.
 */
- (void)onDownloadStatusUpdate:(NSString * _Nonnull)deviceIdentifier response:(TTLServiceDownloadStatusResponse * _Nonnull)downloadStateResponse;

/**
 * Called after the client requested a list of current DTC's recorded by the LINK device.
 * The request is made via [TTLServiceClient requestDTCList:].
 *
 * @see TTLServiceDTCListResponse
 */
- (void)onDTCList:(NSString * _Nonnull)deviceIdentifier response:(TTLServiceDTCListResponse * _Nonnull)response;

/**
 * Called when the state of the engine changes and updates on the engine states are activated.
 *
 * @see TTLServiceEngineStatusResponse
 * @see [TTLServiceClient activateEngineStatusUpdates:]
 */
- (void)onEngineUpdateStatusUpdate:(NSString * _Nonnull)deviceIdentifier response:(TTLServiceEngineStatusResponse * _Nonnull) response;

/**
 * Called on every status update of an ongoing firmware update.
 *
 * @see [TTLServiceEventDelegate onFirmwareUpdateAvailable:response:]
 * @see [TTLServiceClient updateFirmware:deviceIdentifier:]
 * @see TTLServiceFirmwareUpdateStatusResponse
 */
- (void)onFirmwareUpdateStatusUpdate:(NSString * _Nonnull)deviceIdentifier response:(TTLServiceFirmwareUpdateStatusResponse * _Nonnull)response;

/**
 * Called when a firmware update is available on the server.
 *
 * TTLServiceFirmwareUpdateAvailableResponse contains the required data for creating a TTLServiceFirmwareUpdateParameter. It also
 * provides some convenience methods to do that. With TTLServiceFirmwareUpdateParameter you can call [TTLServiceClient updateFirmware:deviceIdentifier:]
 * to start the firmware update.
 *
 * The status and progress of the firmware update is reported via [TTLServiceEventDelegate onFirmwareUpdateStatusUpdate:response:].
 *
 * @see [TTLServiceEventDelegate onFirmwareUpdateStatusUpdate:response:]
 * @see [TTLServiceClient updateFirmware:deviceIdentifier:]
 * @see TTLServiceFirmwareUpdateAvailableResponse
 */
- (void)onFirmwareUpdateAvailable:(NSString * _Nonnull)deviceIdentifier response:(TTLServiceFirmwareUpdateAvailableResponse * _Nonnull)response;

/** Called after the client tried to changed the odometer in the dongle via [TTLServiceClient changeOdometer:deviceIdentifier:].
 *
 * @see TTLServiceOdometerChangeResponse
 */
- (void)onOdometerChange:(NSString * _Nonnull)deviceIdentifier response:(TTLServiceOdometerChangeResponse * _Nonnull)response;

/** Convenience callback to recognize if a connection to the backend service failed.
 *
 * This event indicates that a connection to the backend service failed. The reason for the failure is transported via
 * `TTLServiceServerConnectionFailedReason`. This is not called if a successful established connections goes down for
 * whatever reason. So when [TTLServiceEventDelegate onServerConnectionSuccessful:] was called, this won't be called
 * for the current attempt to connect.
 *
 * @see TTLServiceServerConnectionFailedReason
 */
- (void)onServerConnectionFailed:(NSString * _Nonnull)deviceIdentifier reason:(TTLServiceServerConnectionFailedReason * _Nonnull)reason;

/** Convenience callback to recognize if a connection to the backend service was established successfully.
 *
 * This event indicates that a connection was established successfully and that an initial handshake was
 * successful as well. However, be aware that a disconnect can happen at any time. Currently a disconnect is
 * populated vi [TTLServiceEventDelegate onServiceState:]
 *
 * This is also called, when a connection is in place and [TTLServiceClient connectToServer:] is called.
 */
- (void)onServerConnectionSuccessful:(NSString * _Nonnull)deviceIdentifier;

/** Called when the service state changes or the state of the service was requested via
 *  [TTLServiceClient requestServiceState].
 *
 * The System preserves a state of basically all assets connected to the smart phone. This means the LINK devices and
 * the UBI backend server. See TTLServiceStateResponse for more concrete description what the state is.
 *
 * Since the system works LINK device centric, a separate state is provided for every LINK known to the Application.
 *
 * The system always delivers states for all know LINK devices, even if there is no change. It is up to the
 * application to figure out what has changed.
 *
 *  @param serviceStates An array of TTLServiceStateResponse.
 */
- (void)onServiceState:(NSArray<TTLServiceStateResponse *> * _Nonnull)serviceStates;

- (void)onTransmission:(NSString * _Nonnull)deviceIdentifier response:(TTLServiceTransmissionResponse * _Nonnull)response;

@end

